from _typeshed import Incomplete
from connectsensor.client import (
    AsyncSensorClient as AsyncSensorClient,
    SensorClient as SensorClient,
)
from connectsensor.exceptions import (
    KingspanAPIError as KingspanAPIError,
    KingspanDBError as KingspanDBError,
    KingspanInvalidCredentialsError as KingspanInvalidCredentialsError,
)

__version__: Incomplete
