import logging
from connectsensor import APIVersion as APIVersion, KingspanAPIError as KingspanAPIError, KingspanInvalidCredentialsError as KingspanInvalidCredentialsError, SensorClient as SensorClient

class PasswordFilter(logging.Filter):
    def filter(self, record): ...

def enable_debug_logging() -> None: ...
def main() -> None: ...
